#! /usr/bin/env python
# -*- coding:utf-8 -*-

import sys
import time
import glob
import os
import cv
import numpy as np

from PyQt4 import QtCore
from PyQt4.QtGui import QWidget, QImage, QApplication, QPainter
from naoqi import ALProxy
import naoutil


import image_proc as imgp
import rnn_runner2_pb

import vision_definitions

INIT_HOME_POS = "./init_left_pos.dat"
INIT_HOME_POS = "./init_right_pos.dat"
INIT_HOME_POS = "./init_home_pos.dat"

class ImageWidget(QWidget):
    """
    Tiny widget to display camera images from Naoqi.
    """
    def __init__(self, IP, PORT, CameraID, config_file, rnn_file, time_series_id, parent=None):
        """
        Initialization.
        """
        
        QWidget.__init__(self, parent)
        self._image = QImage()
        self.setWindowTitle('Nao Data Recorder')

        self.img_count = 0
        self.img_dirname = ''
        self._imgWidth = 160
        self._imgHeight = 120
        self._windowWidth = 640
        self._windowHeight = 480
        self._cameraID = CameraID
        self.resize(self._windowWidth, self._windowHeight)
        self.setMinimumSize(self._windowWidth, self._windowHeight)
        self.setMaximumSize(self._windowWidth, self._windowHeight)

        self.body_names = ['RShoulderPitch', 'RShoulderRoll', 'RElbowYaw', 'RElbowRoll',
                           'LShoulderPitch', 'LShoulderRoll', 'LElbowYaw', 'LElbowRoll']

        # Parameters for regression
        self._timeseriesid = time_series_id

        self._windowlength = 10
        self._regcount = 50
        self._rhoinit = 0.0001
        self._momentum = 0.9
       
        """
        self._windowlength = 20
        self._regcount = 20
        self._rhoinit = 0.0001
        self._momentum = 0.9
        """

        self._gammabp = 1.0
        self._gammatop = 0.0
        self._beta = 0.0
        

        # Proxy to ALVideoDevice
        self._videoProxy = None
        self._motionProxy = None
        try:
            self._motionProxy = ALProxy('ALMotion', IP, PORT)
        except Exception, e:
            print 'Could not create proxy to ALMotion'
            print 'Error was: ', e

        
        self.joint_limits = []
        for n in self.body_names:
            self.joint_limits.append(self._motionProxy.getLimits(n)[0])


        # Our video module name
        self._imgClient = ''
        
        self.raw_img = cv.CreateImage((self._imgWidth, self._imgHeight), cv.IPL_DEPTH_8U, 3)
        #####self.hsv_img = cv.CreateImage((self._imgWidthm self._imgHeight), cv.IPL_DEPTH_8U, 3)
        self.gaussian_img = cv.CreateImage((self._imgWidth, self._imgHeight), cv.IPL_DEPTH_8U, 3)
       
        self.image_proc = imgp.ImageProc()
        self.image_proc.Initialize(config_file)

        self.red_contour    = None
        self.yellow_contour = None
        self.green_contour  = None
        self.blue_contour   = None
        self.rnn_orbit = None
        self.filename = None

        self._alImage = None

        self._registerImageClient(IP, PORT)

        # ---- 
        self._is_rnn_run = False
        self._time_step = 0
        self._runner = rnn_runner2_pb.RNNRunner()
        self._runner.init(rnn_file, self._windowlength)
        ###print 'time_series_id', time_series_id
        self._runner.set_time_series_id(time_series_id)
        self._in_state = self._runner.in_state()
        #self._in_state = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
        #                  0.0, 0.0, 0.0]
        self._out_state_queue = []
        for _ in xrange(self._runner.delay_length()):

            self._out_state_queue.append(self._runner.in_state())

        self.startTimer(80)

    def image_process(self):
        rgb_img = cv.CreateImageHeader((self._imgWidth, self._imgHeight), cv.IPL_DEPTH_8U, 3)
        cv.SetData(rgb_img, self._alImage[6])
        cv.CvtColor(rgb_img, self.raw_img, cv.CV_RGB2BGR)
        #####cv.CvtColor(rgb_img, self.hsv_img, cv.CV_RGB2HSV)
        cv.Smooth(self.raw_img, self.gaussian_img, cv.CV_GAUSSIAN, 7, 0)

        self.image_proc.Process(self.gaussian_img)

    def _registerImageClient(self, IP, PORT):
        """
        Register our video module to the robot
        """
       
        try:
            self._videoProxy = ALProxy('ALVideoDevice', IP, PORT)
        except Exception, e:
            print 'Could not create proxy to ALVideoDevice'
            print 'Error was: ', e
        #resolution = vision_definitions.kQVGA  # 320 * 240
        resolution = vision_definitions.kQQVGA # 160 * 120

        colorSpace = vision_definitions.kRGBColorSpace
        fps = 100 # (ms)

        self._imgClient = self._videoProxy.subscribeCamera('_client', 1, resolution, colorSpace, fps)

# 	Select camera
        self._videoProxy.setCameraParameter('_client', vision_definitions.kCameraAutoWhiteBalanceID, 1)
        self._videoProxy.setCameraParameter('_client', vision_definitions.kCameraAutoExpositionID, 1)
        self._videoProxy.setCameraParameterToDefault('_client', vision_definitions.kCameraContrastID)
        self._videoProxy.setCameraParameterToDefault('_client', vision_definitions.kCameraSaturationID)

        self._videoProxy.setCameraParameter('_client', 22, 2) # Auto Exposure Algorithm

    def _unregisterImageClient(self):
        """
        Unregister out naoqi video module.
        """
        if self._imgClient != '':
            self._videoProxy.unsubscribe(self._imgClient)

    def paintEvent(self, event):
        """
        Draw the QImage on screen.
        """
        x_scale = float(self._windowWidth) / self._imgWidth
        y_scale = float(self._windowHeight) / self._imgHeight
        area_scale = 200.0

        painter = QPainter(self)
        painter.drawImage(painter.viewport(), self._image)

        # ----- 

        if self._is_rnn_run:
            painter.setPen(QtCore.Qt.red)
            painter.drawText(10, 20, 'Running')

        painter.setPen(QtCore.Qt.black)
        self.red_contour = self.image_proc.GetContourData(0)
        painter.setBrush(QtCore.Qt.red)
        size = (self.red_contour.area - self.image_proc.GetAreaMin()) / float(self.image_proc.GetAreaMax() - self.image_proc.GetAreaMin()) * area_scale
        painter.drawEllipse(int(self.red_contour.x * x_scale - size / 2), int(self.red_contour.y * y_scale - size / 2),
                            int(size), int(size))

        self.yellow_contour = self.image_proc.GetContourData(1)        
        painter.setBrush(QtCore.Qt.yellow)
        size = (self.yellow_contour.area - self.image_proc.GetAreaMin()) / float(self.image_proc.GetAreaMax() - self.image_proc.GetAreaMin()) * area_scale
        painter.drawEllipse(int(self.yellow_contour.x * x_scale - size / 2), int(self.yellow_contour.y * y_scale - size / 2),
                            int(size), int(size))

        self.green_contour = self.image_proc.GetContourData(2)
        painter.setBrush(QtCore.Qt.green)
        size = (self.green_contour.area - self.image_proc.GetAreaMin()) / float(self.image_proc.GetAreaMax() - self.image_proc.GetAreaMin()) * area_scale
        painter.drawEllipse(int(self.green_contour.x * x_scale - size / 2), int(self.green_contour.y * y_scale - size / 2),
                            int(size), int(size))

        self.blue_contour = self.image_proc.GetContourData(3)
        painter.setBrush(QtCore.Qt.blue)
        size = (self.blue_contour.area - self.image_proc.GetAreaMin()) / float(self.image_proc.GetAreaMax() - self.image_proc.GetAreaMin()) * area_scale
        painter.drawEllipse(int(self.blue_contour.x * x_scale - size / 2), int(self.blue_contour.y * y_scale - size / 2),
                            int(size), int(size))

    def mouseReleaseEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            x_scale = float(self._imgWidth) / self._windowWidth
            y_scale = float(self._imgHeight) / self._windowHeight
            pos = [int(event.x() * x_scale), int(event.y() * y_scale)]
            print ('pos: ' + str(pos) + 
                   '\tBGR: ' + str(cv.Get2D(self.raw_img, pos[1], pos[0])[0:3]))
        else:
            # -----
            #"""
            if self.rnn_orbit == None:
                self.img_count = 0
                count = len(glob.glob('generate*.log'))
                self.filename = 'generate' + str(count).rjust(6, '0') + '.log'
                self.rnn_orbit = []
            else:
                np.savetxt(self.filename, self.rnn_orbit, fmt = "%.6f", delimiter = "\t")
                self.img_count = 0
                self.rnn_orbit = None
            #"""
            if not self._is_rnn_run:
                self.init_pos(INIT_HOME_POS)
                print 'Start running RNN.'

                self._is_rnn_run = True
            else:
                print 'Stop RNN.'
                self._is_rnn_run = False

    def recording(self):
        if self.file != None:
            joint_angles = self._motionProxy.getAngles(self.body_names, True)
            for i in xrange(len(self.body_names)):
                joint_angles[i] = (joint_angles[i] - self.joint_limits[i][0]) / (self.joint_limits[i][1] - self.joint_limits[i][0]) * 1.6 - 0.8
                self.file.write(str(joint_angles[i]) + '\t')

            scales = [self.image_proc.GetXMax() - self.image_proc.GetXMin(),
                      self.image_proc.GetYMax() - self.image_proc.GetYMin(),
                      self.image_proc.GetAreaMax() - self.image_proc.GetAreaMin()]

            contour_data = [self.red_contour.x - self.image_proc.GetXMin(),
                            self.red_contour.y - self.image_proc.GetYMin(),
                            self.red_contour.area - self.image_proc.GetAreaMin()]
            for i in xrange( len(contour_data) ):
                data = float(contour_data[i]) / scales[i] * 1.6 - 0.8
#                self.file.write(str(data) + '\t')

            contour_data = [self.yellow_contour.x - self.image_proc.GetXMin(),
                            self.yellow_contour.y - self.image_proc.GetYMin(),
                            self.yellow_contour.area - self.image_proc.GetAreaMin()]
            for i in xrange( len(contour_data) ):
                data = float(contour_data[i]) / scales[i] * 1.6 - 0.8
                self.file.write(str(data) + '\t')

            contour_data = [self.green_contour.x - self.image_proc.GetXMin(),
                            self.green_contour.y - self.image_proc.GetYMin(),
                            self.green_contour.area - self.image_proc.GetAreaMin()]
            for i in xrange( len(contour_data) ):
                data = float(contour_data[i]) / scales[i] * 1.6 - 0.8
#                self.file.write(str(data) + '\t')

            contour_data = [self.blue_contour.x - self.image_proc.GetXMin(),
                            self.blue_contour.y - self.image_proc.GetYMin(),
                            self.blue_contour.area - self.image_proc.GetAreaMin()]
            for i in xrange( len(contour_data) ):
                data = float(contour_data[i]) / scales[i] * 1.6 - 0.8
#                self.file.write(str(data) + '\t')
            self.file.write('\n')

            img_filename = self.img_dirname + '/img' + str(self.img_count).rjust(6, '0') + '.jpg'
            self.img_count += 1
            cv.SaveImage(img_filename, self.raw_img)

    
    def run_rnn(self):
        if self._is_rnn_run:
            
            try:
                index = len(self.body_names)
                scales = [self.image_proc.GetXMax() - self.image_proc.GetXMin(),
                          self.image_proc.GetYMax() - self.image_proc.GetYMin()]
                          #self.image_proc.GetAreaMax() - self.image_proc.GetAreaMin()]
                data = [self.yellow_contour.x - self.image_proc.GetXMin(),
                        self.yellow_contour.y - self.image_proc.GetYMin()]
                        #self.yellow_contour.area - self.image_proc.GetAreaMin()]
                for d, s in zip(data, scales):
                    print index
                    self._in_state[index] = float(d) / s * 1.6 - 0.8
                    index += 1

            except Exception, e:
                print e
            
            #if len(self._out_state_queue) >= self._runner.delay_length():
            out_state = self._out_state_queue.pop(0)
            print self._time_step

            self._time_step += 1
            print out_state
            #print self._runner.out_state()
            # vision: open-loop, arm: closed-loop
            self._in_state[:len(self.body_names)] = out_state[:len(self.body_names)]
            # all: closed-loop
            #self._in_state[:] = out_state[:]

            
            # Regression
            self._runner.update(self._timeseriesid, self._time_step, self._in_state, self._regcount, self._rhoinit, self._momentum, self._gammabp, self._gammatop, self._beta)
            #self._runner.update(self._timeseriesid, self._time_step, self._in_state, self._regcount, self._rhoinit, self._momentum)

            out_state = self._runner.out_state()
            self._out_state_queue.append(out_state)
            prev_out_state = self._out_state_queue[0]

            self.rnn_orbit.append(np.r_[self._in_state, self._runner.out_state(),
                                        self._runner.var_state(), self._runner.c_inter_state()])

            #print out_state
            target_angle = []
            #for (val, s) in zip(out_state, scale):
            #    target_angle.append(val * s)
            for i, val in enumerate(prev_out_state[:len(self.body_names)]):
                angle = (val + 0.8) * (self.joint_limits[i][1] - self.joint_limits[i][0]) / 1.6 + \
                        self.joint_limits[i][0]

                target_angle.append(angle)

            print self._runner.c_inter_state()
            #self.smootherProxy.setAngles(self.body_names, target_angle)

            self._motionProxy.setAngles(self.body_names, target_angle, 0.2)
            
            #time.sleep(0.01)

    def _updateImage(self):
        """
        Retrieve a new image from Nao
        """
        self._alImage = self._videoProxy.getImageRemote(self._imgClient)
        #print self._alImage
        self._image = QImage(self._alImage[6],           # Pixel array.
                             self._alImage[0],           # Width.
                             self._alImage[1],           # Height.
                             QImage.Format_RGB888)
        self.image_process()

    def timerEvent(self, event):
        """
        Called periodically. Retrieve a nao image, and update the widget.
        """
        self._updateImage()
        # ----- 
        #self.recording()
        self.run_rnn()
        self.update()

    def __del__(self):
        """
        When the widget is deleted, we unregister out naoqi video module.
        """
        self._unregisterImageClient()

    def init_pos(self, filename):
        file = open(filename, "r")
        init_angles = []

        for line in file:
            init_angles.append(float(line))
            #print init_angles
        pTargetAngles = init_angles
        
        pStiffnessLists = 1.0
	pTimeLists = 1.0
        fractionMaxSpeed = 0.08
        
        pNames = ["Head", 'RArm', 'LArm']

	self._motionProxy.stiffnessInterpolation(pNames, pStiffnessLists, pTimeLists)

        pNames = ["Head", 'RShoulderPitch', 'RShoulderRoll', 'RElbowYaw', 'RElbowRoll', 'RHand',
                           'LShoulderPitch', 'LShoulderRoll', 'LElbowYaw', 'LElbowRoll', 'LHand']
	#self._motionProxy.angleInterpolationWithSpeed(pNames, pTargetAngles, fractionMaxSpeed)
        
        self._motionProxy.angleInterpolation(pNames, pTargetAngles, pTimeLists, True)

        time.sleep(1.0)
        
        pNames = ['RShoulderPitch', 'RShoulderRoll', 'RElbowYaw', 'RElbowRoll',
                  'LShoulderPitch', 'LShoulderRoll', 'LElbowYaw', 'LElbowRoll']
        
        pStiffnessLists = 0.0


if __name__ == '__main__':
    IP = 'nao.local'
    PORT = 9559
    CameraID = 1
    config_file = 'config.txt'

    rnn_file = 'rnn.dat'


    time_series_id = 0 #initial PB value corresponding to a trained sequence (ex.0~2:Attract, 3~5:Left, 6~8:Right, 9~11:Self)

    # Read IP address from first argument if any.
    if len(sys.argv) > 1:
        IP = sys.argv[1]

    # Read CameraID from second argument if any.
    if len(sys.argv) > 2:
        CameraID = int(sys.argv[2])


    app = QApplication(sys.argv)
    #myWidget = ImageWidget(IP, PORT, CameraID, filename)
    myWidget = ImageWidget(IP, PORT, CameraID, config_file,
                           rnn_file, time_series_id)
    myWidget.show()
    sys.exit(app.exec_())
